"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KinesisFirehoseEnvelope = void 0;
const envelope_js_1 = require("./envelope.js");
const index_js_1 = require("../schemas/index.js");
const errors_js_1 = require("../errors.js");
/**
 * Kinesis Firehose Envelope to extract array of Records
 *
 *  The record's data parameter is a base64 encoded string which is parsed into a bytes array,
 *  though it can also be a JSON encoded string.
 *  Regardless of its type it'll be parsed into a BaseModel object.
 *
 *  Note: Records will be parsed the same way so if model is str,
 *  all items in the list will be parsed as str and not as JSON (and vice versa)
 *
 *  https://docs.aws.amazon.com/lambda/latest/dg/services-kinesisfirehose.html
 */
class KinesisFirehoseEnvelope extends envelope_js_1.Envelope {
    static parse(data, schema) {
        const parsedEnvelope = index_js_1.KinesisFirehoseSchema.parse(data);
        return parsedEnvelope.records.map((record) => {
            return super.parse(record.data, schema);
        });
    }
    static safeParse(data, schema) {
        const parsedEnvelope = index_js_1.KinesisFirehoseSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new errors_js_1.ParseError('Failed to parse Kinesis Firehose envelope', parsedEnvelope.error),
                originalEvent: data,
            };
        }
        const parsedRecords = [];
        for (const record of parsedEnvelope.data.records) {
            const parsedData = super.safeParse(record.data, schema);
            if (!parsedData.success) {
                return {
                    success: false,
                    error: new errors_js_1.ParseError('Failed to parse Kinesis Firehose record', parsedData.error),
                    originalEvent: data,
                };
            }
            parsedRecords.push(parsedData.data);
        }
        return {
            success: true,
            data: parsedRecords,
        };
    }
}
exports.KinesisFirehoseEnvelope = KinesisFirehoseEnvelope;
