"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIGatewayCert = exports.APIGatewayProxyEventSchema = void 0;
const zod_1 = require("zod");
const APIGatewayCert = zod_1.z.object({
    clientCertPem: zod_1.z.string(),
    subjectDN: zod_1.z.string(),
    issuerDN: zod_1.z.string(),
    serialNumber: zod_1.z.string(),
    validity: zod_1.z.object({
        notBefore: zod_1.z.string(),
        notAfter: zod_1.z.string(),
    }),
});
exports.APIGatewayCert = APIGatewayCert;
const APIGatewayEventIdentity = zod_1.z.object({
    accessKey: zod_1.z.string().nullish(),
    accountId: zod_1.z.string().nullish(),
    apiKey: zod_1.z.string().nullish(),
    apiKeyId: zod_1.z.string().nullish(),
    caller: zod_1.z.string().nullish(),
    cognitoAuthenticationProvider: zod_1.z.string().nullish(),
    cognitoAuthenticationType: zod_1.z.string().nullish(),
    cognitoIdentityId: zod_1.z.string().nullish(),
    cognitoIdentityPoolId: zod_1.z.string().nullish(),
    principalOrgId: zod_1.z.string().nullish(),
    /**
     * When invoking the API Gateway REST API using the Test Invoke feature,
     * the sourceIp is hardcoded to `test-invoke-source-ip`. This is a stopgap
     * solution to allow customers to test their API and have successful parsing.
     *
     * See aws-powertools/powertools-lambda-python#1562 for more information.
     */
    sourceIp: zod_1.z
        .union([zod_1.z.string().ip(), zod_1.z.literal('test-invoke-source-ip')])
        .optional(),
    user: zod_1.z.string().nullish(),
    userAgent: zod_1.z.string().nullish(),
    userArn: zod_1.z.string().nullish(),
    clientCert: APIGatewayCert.nullish(),
});
const APIGatewayEventRequestContext = zod_1.z
    .object({
    accountId: zod_1.z.string(),
    apiId: zod_1.z.string(),
    authorizer: zod_1.z
        .object({
        claims: zod_1.z.record(zod_1.z.string(), zod_1.z.any()).nullish(),
        scopes: zod_1.z.array(zod_1.z.string()).nullish(),
    })
        .nullish(),
    stage: zod_1.z.string(),
    protocol: zod_1.z.string(),
    identity: APIGatewayEventIdentity,
    requestId: zod_1.z.string(),
    requestTime: zod_1.z.string(),
    requestTimeEpoch: zod_1.z.number(),
    resourceId: zod_1.z.string().nullish(),
    resourcePath: zod_1.z.string(),
    domainName: zod_1.z.string().nullish(),
    domainPrefix: zod_1.z.string().nullish(),
    extendedRequestId: zod_1.z.string().nullish(),
    httpMethod: zod_1.z.enum([
        'GET',
        'POST',
        'PUT',
        'PATCH',
        'DELETE',
        'HEAD',
        'OPTIONS',
    ]),
    path: zod_1.z.string(),
    connectedAt: zod_1.z.number().nullish(),
    connectionId: zod_1.z.string().nullish(),
    eventType: zod_1.z.enum(['CONNECT', 'MESSAGE', 'DISCONNECT']).nullish(),
    messageDirection: zod_1.z.string().nullish(),
    messageId: zod_1.z.string().nullish(),
    routeKey: zod_1.z.string().nullish(),
    operationName: zod_1.z.string().nullish(),
})
    .refine((input) => {
    return (!input.messageId || (input.messageId && input.eventType === 'MESSAGE'));
}, {
    message: 'messageId is available only when `eventType` is MESSAGE',
});
const APIGatewayProxyEventSchema = zod_1.z.object({
    version: zod_1.z.string().optional(),
    authorizationToken: zod_1.z.string().optional(),
    identitySource: zod_1.z.string().optional(),
    methodArn: zod_1.z.string().optional(),
    type: zod_1.z.enum(['TOKEN', 'REQUEST']).optional(),
    resource: zod_1.z.string(),
    path: zod_1.z.string(),
    httpMethod: zod_1.z.enum([
        'GET',
        'POST',
        'PUT',
        'PATCH',
        'DELETE',
        'HEAD',
        'OPTIONS',
    ]),
    headers: zod_1.z.record(zod_1.z.string()).optional(),
    queryStringParameters: zod_1.z.record(zod_1.z.string()).nullable(),
    multiValueHeaders: zod_1.z.record(zod_1.z.array(zod_1.z.string())).optional(),
    multiValueQueryStringParameters: zod_1.z.record(zod_1.z.array(zod_1.z.string())).nullable(),
    requestContext: APIGatewayEventRequestContext,
    pathParameters: zod_1.z.record(zod_1.z.string()).optional().nullish(),
    stageVariables: zod_1.z.record(zod_1.z.string()).optional().nullish(),
    isBase64Encoded: zod_1.z.boolean().optional(),
    body: zod_1.z.string().nullable(),
});
exports.APIGatewayProxyEventSchema = APIGatewayProxyEventSchema;
